/**
 * $Id: list.h,v 1.8 2007/04/06 09:05:26 superkkt Exp $
 *
 * DCE / THREAD List Thread Header
 *
 * Copyright 2007 Samjung Data Service, Inc.  All rights reserved.
 * 김기태 (superkkt@sds.co.kr)
 */

#ifndef LIST_H
#define LIST_H

#include <pthread.h>

#include "pxe.h"
#include "support.h"

#define MAX_THREAD	512	/* <= MAX of PXEThread type */
#define MAX_DCE		256	/* <= MAX of PXE_DCE type */		

#define MAX_CIRCULAR_QUEUE	512

/* 원형큐 오버플로우시 딜레이 주기 (microseconds) */
#define OVERFLOW_DELAY	10000	/* 1/100 sec */

/* 
 * 필수 쓰레드 리스트 (10 미만의 쓰레드 번호는 PXE의 필수 쓰레드용으로 예약)
 */
#define SD_THREAD       0       /* Sender (실제 존재하는 쓰레드 아님) */
#define CT_THREAD	1	/* Controller Thread */

/*
 * THREAD LIST
 */
typedef struct
{
    int active;                 /* Activation status */
    pthread_t tid;              /* Thread ID */
    pthread_mutex_t mutex;      /* Mutex */
    pthread_cond_t cond;        /* Condition variable */
    int sig_status;             /* Signal status */
    int head;                   /* Queue head index */
    int tail;                   /* Queue tail index */
    void *buf[MAX_CIRCULAR_QUEUE];      /* Data buffer */
    PXELength len[MAX_CIRCULAR_QUEUE];  /* Data buffer size */
} ThreadList;

/*
 * DCE LIST
 */
typedef struct
{
    int active;             /* Connection status */
    char alias[256];        /* Peer Alias Name */
    char ip[16];            /* Peer IP */
    int fd;                 /* Socket descriptor */
    pthread_mutex_t mutex;  /* Socket MUTEX */
} DCEList;

/* Thread Argument */ 
typedef PXEArg ThreadArg;

int list_init(void);

/* 
 * THREAD LIST Functions
 */
int set_thread_list(PXEThread idx, void *(*function)(void *), 
	void *arg, size_t len, int options);
BOOL is_tlist_active(PXEThread idx);
void inactivate_tlist(PXEThread idx);
pthread_t get_tlist_tid(PXEThread idx);

/*
 * DCE LIST Functions
 */
void dlist_mutex_lock(PXE_DCE idx);
void dlist_mutex_unlock(PXE_DCE idx);
int set_dce_list(const char *alias, const char *ip, int fd);
BOOL is_dlist_active(PXE_DCE idx);
void inactivate_dlist(PXE_DCE idx);
int get_dlist_fd(PXE_DCE idx);
char *get_dlist_ip(PXE_DCE idx);
PXE_DCE search_dce_idx(const char *ip);
void set_master_dce(const char *ip, int fd);
int get_dce_list(PXE_DCE_LIST ***ptr);
void free_dce_list(PXE_DCE_LIST **list);
BOOL is_connected_agent(const char *ip, const char *alias);

/*
 * Signal Functions
 */
void send_signal(PXEThread idx);
void recv_signal(PXEThread idx);

/*
 * Circular Queue Functions
 */
int push_queue(PXEThread idx, void *buf);
void pop_queue(PXEThread idx, void **packet, PXELength *len);
int pop_queue_non_block(PXEThread idx, void **packet, PXELength *len);
int pop_queue_timedwait(PXEThread idx, void **packet, PXELength *len, int sec);

#endif
