/**
 * $Id: log.c,v 1.5 2007/04/06 09:05:26 superkkt Exp $
 *
 * Log Functions
 *
 * Copyright 2007 Samjung Data Service, Inc.  All rights reserved.
 * 김기태 (superkkt@sds.co.kr)
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdarg.h>

#include "log.h"
#include "pxe.h"

#define LOG_MAX    8192

/* 로그기록 함수 포인터 */
static LogFunction log_func = NULL;

/**
 * default_log - 기본 로그기록 함수
 *
 * @parm msg: 에러 메세지
 * @return: None
 */
void
default_log(const char *msg)
{
    (void) 0;
}

/**
 * set_log_function - 로그기록 함수 지정
 *
 * @parm log: 로그기록 함수 포인터
 * @return: None
 */
void
set_log_function(LogFunction log)
{
    if (log)
	log_func = log;
    else
	log_func = default_log;
}

/**
 * Log - 로그기록 함수 Wrapping 함수
 *
 * @parm format: Log 포맷
 * @parm ...: Log 포맷에 필요한 가변인자
 * @return: None
 */
void
Log(char *format, ...)
{
    va_list ap;
    char buf[LOG_MAX];

    va_start(ap, format);
    vsnprintf(buf, sizeof(buf), format, ap);
    log_func(buf);
    va_end(ap);
}
