/**
 * $Id: pxe.h,v 1.8 2007/04/06 09:05:26 superkkt Exp $
 *
 * PXE Main Header
 *
 * Copyright 2007 Samjung Data Service, Inc.  All rights reserved.
 * 김기태 (superkkt@sds.co.kr)
 */

#ifndef PXE_H
#define PXE_H

#include <inttypes.h>
#include <sys/types.h>

/*
 * Emblem for valid PXE packet
 */
#define PXE_EMBLEM    0xDB09

/*
 * Thread ID auto assign
 */
#define PXE_AUTO_ASSIGN    -1 

/* 
 * Special IP Address
 */
#define PXE_LOCALHOST	"255.255.255.255"
#define PXE_MASTER	"0.0.0.0"

/*
 * Thread Load Options
 */
#define PXE_DETACH_THREAD	1 << 0

/*
 * Byte Order Macros
 */
#define ID_HTON(x)	htonl((x))
#define DCE_HTON(x)	htons((x))
#define THREAD_HTON(x)	htons((x))
#define LENGTH_HTON(x)	htonl((x))

#define ID_NTOH(x)	ntohl((x))
#define DCE_NTOH(x)	ntohs((x))
#define THREAD_NTOH(x)	ntohs((x))
#define LENGTH_NTOH(x)	ntohl((x))

/*
 * Types
 */
typedef void (*LogFunction)(const char *msg);
typedef int16_t PXE_DCE;
typedef int16_t PXEThread;
typedef int32_t PXELength;

/*
 * PXE Packet Protocol
 */
#define PXE_PACKET_SIZE   28   
typedef struct 
{
    uint32_t emblem;            /* Emblem for valid PXE packet */
#define DATA_ID    1 << 0
#define PING_ID    1 << 1
    uint32_t id;                /* Packet ID */
    uint8_t dst_ip[4];          /* Destination IP */
    uint8_t src_ip[4];          /* Source IP */
    PXE_DCE dst_dce;            /* Destination DCE */
    PXE_DCE src_dce;            /* Source DCE */
    PXEThread dst_thread;       /* Destination Thread */
    PXEThread src_thread;       /* Source Thread */
    PXELength length;           /* Packet length except PXE header */
} PXEPacket;

/*
 * Thread Argument
 */
typedef struct
{
    int idx;            /* THREAD LIST Index */
    char extra_arg[1];  /* User Argument */
} PXEArg;

/*
 * PXE Handler Options 
 */
#define PXE_NON_BLOCK	1 << 0
#define PXE_TIMEDWAIT   1 << 1

/* 
 * PXE Handler
 */
typedef struct
{
    unsigned char peer_ip[4];
    PXE_DCE peer_dce;
    PXEThread peer_thread;
    PXEThread my_thread;
    int options;
} PXE;

typedef struct
{
    char *ip;
    char *alias;
} PXE_DCE_LIST;

/*
 * Function Prototypes
 */
int pxe_init(LogFunction log);
int pxe_master(int port);
int pxe_agent(const char *alias, const char *ip, int port, int delay);
int pxe_load_thread(PXEThread idx, void *(*function)(void *), 
	void *arg, size_t len, int options);

PXE *pxe_open(PXEArg *arg);
void *pxe_get_argument(PXEArg *arg);
PXEThread pxe_get_src_thread(PXE *pxe);
void pxe_get_src_ip(PXE *pxe, char *ip, size_t len);
int pxe_write(PXE *pxe, const void *buf, size_t len);
PXELength pxe_read(PXE *pxe, void **ptr);
PXELength pxe_read_timedwait(PXE *pxe, void **ptr, int timeout);
int pxe_is_from_external(PXE *pxe);
void pxe_free(void *ptr);
void pxe_close(PXE *pxe);
void pxe_sleep(void);
int pxe_get_dce_list(PXE_DCE_LIST ***list);
void pxe_free_dce_list(PXE_DCE_LIST **list);

int pxe_get_options(PXE *pxe);
void pxe_set_options(PXE *pxe, int options);
int pxe_set_destination(PXE *pxe, const char *ip, PXEThread dst_thread);

#endif
